/*----------------------------------------------------------------------- 
 * 
 * File Name: RealFFTTest.c
 * 
 * Author: Creighton, J. D. E.
 * 
 * Revision: $Id: RealFFTTest.c$
 * 
 *----------------------------------------------------------------------- 
 * 
 * NAME 
 *   main()
 *
 * SYNOPSIS 
 * 
 * DESCRIPTION 
 *   Tests real FFT functions.
 * 
 * DIAGNOSTICS
 * 
 * CALLS
 * 
 * NOTES
 *----------------------------------------------------------------------- 
 * 
 * REVISION HISTORY 
 * 
 * $Log: RealFFTTest.c$
 * 
 *-----------------------------------------------------------------------
 */

#ifndef _STDIO_H
#include <stdio.h>
#ifndef _STDIO_H
#define _STDIO_H
#endif
#endif

#ifndef _MATH_H
#include <math.h>
#ifndef _MATH_H
#define _MATH_H
#endif
#endif

#ifndef _LALSTDLIB_H
#include "LALStdlib.h"
#ifndef _LALSTDLIB_H
#define _LALSTDLIB_H
#endif
#endif

#ifndef _SEQFACTORIES_H
#include "SeqFactories.h"
#ifndef _SEQFACTORIES_H
#define _SEQFACTORIES_H
#endif
#endif

#ifndef _REALFFT_H
#include "RealFFT.h"
#ifndef _REALFFT_H
#define _REALFFT_H
#endif
#endif

#ifndef _VECTOROPS_H
#include "VectorOps.h"
#ifndef _VECTOROPS_H
#define _VECTOROPS_H
#endif
#endif


int debuglevel = 2;

static volatile const CHAR *MAIN = "$Id: RealFFTTest.c$";

int main()
{
  const INT4 m = 4;   /* example length of sequence of vectors */
  const INT4 n = 32;  /* example vector length */

  Status status; 

  RealFFTPlan            *pfwd = NULL;
  RealFFTPlan            *pinv = NULL;
  REAL4Vector            *hvec = NULL;
  COMPLEX8Vector         *Hvec = NULL;
  REAL4Vector            *Pvec = NULL;
  REAL4VectorSequence    *hseq = NULL;
  COMPLEX8VectorSequence *Hseq = NULL;
  REAL4VectorSequence    *Pseq = NULL;
  CreateVectorSequenceIn  seqinp;
  CreateVectorSequenceIn  seqout;

  INT4 i;
  INT4 j;

  seqinp.length       = m;
  seqinp.vectorLength = n;
  seqout.length       = m;
  seqout.vectorLength = n/2 + 1;

  SCreateVector         (&status, &hvec, n);
  CCreateVector         (&status, &Hvec, n/2 + 1);
  SCreateVector         (&status, &Pvec, n/2 + 1);
  SCreateVectorSequence (&status, &hseq, &seqinp);
  CCreateVectorSequence (&status, &Hseq, &seqout);
  SCreateVectorSequence (&status, &Pseq, &seqout);

  EstimateFwdRealFFTPlan (&status, &pfwd, n);
  EstimateInvRealFFTPlan (&status, &pinv, n);
  DestroyRealFFTPlan     (&status, &pfwd);
  DestroyRealFFTPlan     (&status, &pinv);
  MeasureFwdRealFFTPlan  (&status, &pfwd, n);
  MeasureInvRealFFTPlan  (&status, &pinv, n);

  /* assign data ... */
  printf ("\nInitial data:\n");
  for (i = 0; i < n; ++i)
  {
    REAL4 data = cos(7*i) - 1;
    hvec->data[i] = data;
    for (j = 0; j < m; ++j)
    {
      hseq->data[i + j*n] = data;
      printf ("% 9.3f\t", data);
      data += 1;
    }
    printf ("\n");
  }

  /* perform FFTs */

  printf ("\nSingle Forward FFT:\n");
  FwdRealFFT (&status, Hvec, hvec, pfwd);
  for (i = 0; i < Hvec->length; ++i)
    printf ("(% 9.3f, % 9.3f)\n", Hvec->data[i].re, Hvec->data[i].im);

  printf ("\nSingle Forward FFT Power:\n");
  RealPowerSpectrum (&status, Pvec, hvec, pfwd);
  for (i = 0; i < Pvec->length; ++i)
    printf ("%12.3f\n", Pvec->data[i]);

  printf ("\nSingle Inverse FFT:\n");
  InvRealFFT (&status, hvec, Hvec, pinv);
  for (i = 0; i < hvec->length; ++i)
    printf ("% 9.3f\n", hvec->data[i]/n);

  printf ("\nMultiple Forward FFT:\n");
  FwdRealSequenceFFT (&status, Hseq, hseq, pfwd);
  for (i = 0; i < Hseq->vectorLength; ++i)
  {
    for (j = 0; j < Hseq->length; ++j)
      printf ("(% 9.3f, % 9.3f)\t",
                    Hseq->data[i + j*Hseq->vectorLength].re,
                    Hseq->data[i + j*Hseq->vectorLength].im);
    printf ("\n");
  }

  printf ("\nMultiple Forward FFT Power:\n");
  RealSequencePowerSpectrum (&status, Pseq, hseq, pfwd);
  for (i = 0; i < Pseq->vectorLength; ++i)
  {
    for (j = 0; j < Pseq->length; ++j)
      printf ("%12.3f\t", Pseq->data[i + j*Pseq->vectorLength]);
    printf ("\n");
  }

  printf ("\nMultiple Inverse FFT:\n");
  InvRealSequenceFFT (&status, hseq, Hseq, pinv);
  for (i = 0; i < hseq->vectorLength; ++i)
  {
    for (j = 0; j < hseq->length; ++j)
      printf ("% 9.3f\t", hseq->data[i + j*hseq->vectorLength]/n);
    printf ("\n");
  }

  /* destroy plans, vectors, and sequences */
  DestroyRealFFTPlan     (&status, &pfwd);
  DestroyRealFFTPlan     (&status, &pinv);
  SDestroyVector         (&status, &hvec);
  CDestroyVector         (&status, &Hvec);
  SDestroyVector         (&status, &Pvec);
  SDestroyVectorSequence (&status, &hseq);
  CDestroyVectorSequence (&status, &Hseq);
  SDestroyVectorSequence (&status, &Pseq);

  return 0;
}
